# flake8: noqa E401,E403
# this file exists only for backwards compatability
# it allow the use of `import readchar.key`

from . import platform


if platform.startswith(("linux", "darwin", "freebsd")):
    from ._posix_key import *
elif platform in ("win32", "cygwin"):
    from ._win_key import *
else:
    raise NotImplementedError(f"The platform {platform} is not supported yet")



from ._base_key import *


# Windows uses scan codes for extended characters. This dictionary
# translates the second half of the scan codes of special Keys
# into the corresponding variable used by readchar.
#
# for windows scan codes see:
#   https://msdn.microsoft.com/en-us/library/aa299374
#      or
#   https://www.freepascal.org/docs-html/rtl/keyboard/kbdscancode.html

# common
BACKSPACE = "\x08"

# cursors
UP = "\x00\x48"
DOWN = "\x00\x50"
LEFT = "\x00\x4b"
RIGHT = "\x00\x4d"

# navigation keys
INSERT = "\x00\x52"
SUPR = "\x00\x53"
HOME = "\x00\x47"
END = "\x00\x4f"
PAGE_UP = "\x00\x49"
PAGE_DOWN = "\x00\x51"

# funcion keys
F1 = "\x00\x3b"
F2 = "\x00\x3c"
F3 = "\x00\x3d"
F4 = "\x00\x3e"
F5 = "\x00\x3f"
F6 = "\x00\x40"
F7 = "\x00\x41"
F8 = "\x00\x42"
F9 = "\x00\x43"
F10 = "\x00\x44"
F11 = "\x00\x85"  # only in second source
F12 = "\x00\x86"  # only in second source

# other
ESC_2 = "\x00\x01"
ENTER_2 = "\x00\x1c"

# don't have table entries for...
# ALT_[A-Z]
# CTRL_ALT_[A-Z],
# CTRL_ALT_SUPR,
# CTRL-F1


# aliases
ENTER = CR
DELETE = SUPR



import sys
import termios

from ._config import config


# Initially taken from:
# http://code.activestate.com/recipes/134892/
# Thanks to Danny Yoo
# more infos from:
# https://gist.github.com/michelbl/efda48b19d3e587685e3441a74457024
# Thanks to Michel Blancard
def readchar() -> str:
    """Reads a single character from the input stream.
    Blocks until a character is available."""

    fd = sys.stdin.fileno()
    old_settings = termios.tcgetattr(fd)
    term = termios.tcgetattr(fd)
    try:
        term[3] &= ~(termios.ICANON | termios.ECHO | termios.IGNBRK | termios.BRKINT)
        termios.tcsetattr(fd, termios.TCSAFLUSH, term)

        ch = sys.stdin.read(1)
    finally:
        termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
    return ch


def readkey() -> str:
    """Get a keypress. If an escaped key is pressed, the full sequence is
    read and returned as noted in `_posix_key.py`."""

    c1 = readchar()

    if c1 in config.INTERRUPT_KEYS:
        raise KeyboardInterrupt

    if c1 != "\x1B":
        return c1

    c2 = readchar()
    if c2 not in "\x4F\x5B":
        return c1 + c2

    c3 = readchar()
    if c3 not in "\x31\x32\x33\x35\x36":
        return c1 + c2 + c3

    c4 = readchar()
    if c4 not in "\x30\x31\x33\x34\x35\x37\x38\x39":
        return c1 + c2 + c3 + c4

    c5 = readchar()
    return c1 + c2 + c3 + c4 + c5



from typing import List

from . import _base_key as key


class config:
    """Static class that containes Constants used throughout the libary.
    You can directly use the class-attributes, do not create instances of it!"""

    def __new__(cls):
        raise SyntaxError("you can't create instances of this class")

    INTERRUPT_KEYS: List[str] = [key.CTRL_C]
